<?php

// $PdfLink
// $BibLink
// $UrlLink

class BibtexEntry {
  var $values = array();
  var $bibfile;
  var $entryname;
  var $entrytype;

  function BibtexEntry($bibfile, $entryname)
  {
    $this->bibfile = $bibfile;
    $this->entryname = $entryname;
  }

  function getAuthors()
  {
    $aut = $this->get('AUTHOR');
    if ($aut == FALSE) return FALSE;
    $aut = explode(" and ", $aut);

    $ret = "";

    for ($i = 0; $i < count($aut); $i++)
      {
	$ret = $ret . $aut[$i];
	if ($i == count($aut) - 2) $ret = $ret . " and ";
	else if ($i < count($aut) - 2) $ret = $ret . ", ";
      }
    return $ret;
  }

  function getTitle()
  {
    return $this->getFormat('TITLE');
  }

  function getAbstract()
  {
    return $this->get('ABSTRACT');
  }

  function getComment()
  {
    return $this->get('COMMENT');
  }

  function getPages()
  {
    $pages = $this->get('PAGES');
    if ($pages)
    {
      $found = strpos($pages, "--");
      if (found) return str_replace("--", "-", $pages);
      else return $pages;
    }
    return "";
  }

  function get($field)
  {
    return $this->values[$field];
  }


  function getFormat($field)
  {
    $ret = $this->get($field);
    if ($ret)
    {
      $ret = str_replace("{", "", $ret);
      $ret = str_replace("}", "", $ret);
    }
    return $ret;
  }

  function getCompleteEntryUrl()
  {
    global $pagename;
    return FmtPageName('$PageUrl', $pagename) . "?action=bibentry&bibfile=" . $this->bibfile . "&bibref=" . $this->entryname;
  }

  function getPreString($dourl = true)
  {
    global $pagename;

    if ($dourl)
      $ret = "''[[" . $this->getCompleteEntryUrl() . " " . $this->getTitle() . "]]" . "''. " . $this->getAuthors();
    else
      $ret = "''" . $this->getTitle() . "''. " . $this->getAuthors();
    return $ret;
  }

  function getPostString($dourl = true)
  {
    $ret = "";
    $year = $this->get("YEAR");
    if ($year)
    {
      $ret = $ret . ", ";
      $month = $this->get("MONTH");
      if ($month)
      {
	$ret = $ret . $month . " ";
      }
      $ret = $ret . $year;
    }
    $ret = $ret . ".";

    if (!$dourl)
    {
      $url = $this->get("URL");
      if ($url)
	{
	  $ret = $ret . " [ [[" . $url . " URL]] ]";
	}
    }

    $pdf = $this->get("PDF");
    if ($pdf)
    {
      $ret = $ret . " [[Attach:" . $pdf . " Attach:pdf.gif]]";
    }

    if ($dourl) $ret = $ret . " [ [[" . $this->getCompleteEntryUrl() . "#" . $this->entryname . "Bib BIB]] ]";

    return $ret;
  }

  function cite()
  {
    $ret = "[[[" . $this->getCompleteEntryUrl() . " " . $this->entryname . "]]]";
    return $ret;
  }

  function getBibEntry()
  {
    $ret = $ret . "@@@" . $this->entrytype . " {\\\\\n";

    while (list($key, $value)=each($this->values))
    {
      if ($key == "COMMENT" || $key == "ABSTRACT" || $key == "KEYWORDS" || $key == "PDF") continue;
      $ret = $ret . "    " . $key . " = { " . $value . " },\\\\\n";
    }
    $ret = $ret . "}@@\n";

    return $ret;
  }

  function getCompleteEntry()
  {
    $ret = "[[#" . $this->entryname . "]]\n!!!" . $this->entryname . "\n";
    $ret = $ret . $this->getSummary(false) . "\n";
    $abstract = $this->getAbstract();
    if ($abstract)
    {
      $ret = $ret . "\n'''Abstract:'''\n->" . $abstract . "\n";
    }
    $comment = $this->getComment();
    if ($comment)
    {
      $ret = $ret . "\n'''Comment:'''\n->" . $comment . "\n";
    }
    $ret = $ret . "[[#" . $this->entryname . "Bib]]\n";
    $ret = $ret . "\n'''Bibtex entry:'''\n->" . $this->getBibEntry() . "\n";
    return $ret;
  }

  function getSolePageEntry()
  {
    $ret = "!" . $this->entryname . "\n";
    $ret = $ret . "\n!!!Summary\n";
    $ret = $ret . $this->getSummary(false) . "\n";
    $abstract = $this->getAbstract();
    if ($abstract)
    {
      $ret = $ret . "\n!!!Abstract\n" . $abstract . "\n";
    }
    $comment = $this->getComment();
    if ($comment)
    {
      $ret = $ret . "\n!!!Comment\n" . $comment . "\n";
    }
    $ret = $ret . "[[#" . $this->entryname . "Bib]]\n";
    $ret = $ret . "\n!!!Bibtex entry\n" . $this->getBibEntry() . "\n";
    return $ret;
  }
}

class PhdThesis extends BibtexEntry {
  function PhdThesis($bibfile, $entryname)
  {
    parent::BibtexEntry($bibfile, $entryname);
    $this->entrytype = "PHDTHESIS";
  }

  function getSummary($dourl = true)
  {
    $ret = parent::getPreString($dourl);
    $ret = $ret . ". PhD thesis";
    $school = parent::get("SCHOOL");
    if ($school)
    {
      $ret = $ret . ", ''" . $school . "''";
    }
    return $ret . parent::getPostString($dourl);
  }
}

class MasterThesis extends BibtexEntry {
  function MasterThesis($bibfile, $entryname)
  {
    parent::BibtexEntry($bibfile, $entryname);
    $this->entrytype = "MASTERSTHESIS";
  }

  function getSummary($dourl = true)
  {
    $ret = parent::getPreString($dourl);
    $ret = $ret . ". Master's thesis";
    $school = parent::get("SCHOOL");
    if ($school)
    {
      $ret = $ret . ", ''" . $school . "''";
    }
    return $ret . parent::getPostString($dourl);
  }
}

class TechReport extends BibtexEntry {
  function TechReport($bibfile, $entryname)
  {
    parent::BibtexEntry($bibfile, $entryname);
    $this->entrytype = "TECHREPORT";
  }

  function getSummary($dourl = true)
  {
    $ret = parent::getPreString($dourl);
    $ret = $ret . ". Technical report";
    $institution = parent::get("INSTITUTION");
    if ($institution)
    {
      $ret = $ret . ", ''" . $institution . "''";
    }
    return $ret . parent::getPostString($dourl);
  }
}

class Article extends BibtexEntry {
  function Article($bibfile, $entryname)
  {
    parent::BibtexEntry($bibfile, $entryname);
    $this->entrytype = "ARTICLE";
  }

  function getSummary($dourl = true)
  {
    $ret = parent::getPreString($dourl);
    $journal = parent::get("JOURNAL");
    if ($journal)
    {
      $ret = $ret . ". ''" . $journal . "''";
      $volume = parent::get("VOLUME");
      if ($volume)
      {
	$ret = $ret . ", " . $volume;
	$number = parent::get("NUMBER");
	if ($number)
	{
	  $ret = $ret . "(" . $number . ")";
	}
	$pages = parent::getPages();
	if ($pages)
	{
	  $ret = $ret . ":" . $pages;
	}
      }
    }
    return $ret . parent::getPostString($dourl);
  }
}

class InProceedings extends BibtexEntry
{
  function InProceedings($bibfile, $entryname)
  {
    parent::BibtexEntry($bibfile, $entryname);
    $this->entrytype = "INPROCEEDINGS";
  }

  function getSummary($dourl = true)
  {
    $ret = parent::getPreString($dourl);
    $booktitle = parent::get("BOOKTITLE");
    if ($booktitle)
    {
      $ret = $ret . ". In " . $booktitle . "";
      $pages = $this->getPages();
      if ($pages) $ret = $ret . ", " . $pages;
      $address = parent::get("ADDRESS");
      if ($address) $ret = $ret . ", " . $address;
      $organization = parent::get("ORGANIZATION");
      if ($organization) $ret = $ret . ". " . $organization;
    }

    return $ret . parent::getPostString($dourl);
  }

  function getPages()
  {
    $pages = parent::getPages();
    if ($pages)
    {
      if (strpos($pages, "-")) return "pages " . $pages;
      else return "page " . $pages;
    }
    return $pages;
  }
}

class InCollection extends BibtexEntry {
  function InCollection($bibfile, $entryname)
  {
    parent::BibtexEntry($bibfile, $entryname);
    $this->entrytype = "INCOLLECTION";
  }

  function getSummary($dourl = true)
  {
    $ret = parent::getPreString($dourl);
    $booktitle = parent::get("BOOKTITLE");
    if ($booktitle)
    {
      $ret = $ret . ". In " . $booktitle . "";
      $pages = $this->getPages();
      if ($pages) $ret = $ret . ", " . $pages . ".";
      $publisher = parent::get("PUBLISHER");
      if ($publisher) $ret = $ret . $publisher . ", ";
    }

  function getPages()
  {
    $pages = parent::getPages();
    if ($pages)
    {
      if (strpos($pages, "-")) return "pages " . $pages;
      else return "page " . $pages;
    }
    return $pages;
  }

    return $ret . parent::getPostString($dourl);
    
  }
}

class Book extends BibtexEntry {
  function Book($bibfile, $entryname)
  {
    parent::BibtexEntry($bibfile, $entryname);
    $this->entrytype = "INCOLLECTION";
  }

  function getSummary($dourl = true)
  {
    $ret = parent::getPreString($dourl);
    $publisher = parent::get("PUBLISHER");
    if ($publisher) $ret = $ret . $publisher . ", ";
    $ret = $ret . parent::getPostString($dourl);
  }
}

class Misc extends BibtexEntry {
  function Misc($bibfile, $entryname)
  {
    parent::BibtexEntry($bibfile, $entryname);
    $this->entrytype = "MISC";
  }

  function getSummary($dourl = true)
  {
    return parent::getPreString($dourl) . parent::getPostString($dourl);
  }
}

$DoubleBrackets['/\\{\\[(.*?),(.*?)\\]\\}/e'] = "BibCite('$1', '$2')";
$BrowseDirectives['/\\bbibtexsummary:\\[(.*?),(.*?)\\]/e'] = "BibSummary('$1', '$2')";
$BrowseDirectives['/\\bbibtexcomplete:\\[(.*?),(.*?)\\]/e'] = "CompleteBibEntry('$1', '$2')";

SDV($HandleActions['bibentry'],'HandleBibEntry');

function HandleBibEntry($pagename)
{
  global $HTTP_GET_VARS, $PageStartFmt, $PageEndFmt, $ScriptUrl, $bibentry, $bibfile, $bibref;
  $bibfile = $HTTP_GET_VARS['bibfile'];
  $bibref = $HTTP_GET_VARS['bibref'];
  SDV($ScriptUrl, FmtPageName('$PageUrl', $pagename));

  $bibentry = GetEntry($bibfile, $bibref);

  $page = array('timefmt'=>@$GLOBALS['CurrentTime'], 
		'author'=>@$GLOBALS['Author']);
  SetPageVars($pagename,$page, (bibentry != false ? $bibentry->getTitle() : ""));

  SDV($HandleBibtexFmt,array(&$PageStartFmt,
    'function:PrintCompleteEntry',&$PageEndFmt));
  PrintFmt($pagename,$HandleBibtexFmt);
}

function PrintCompleteEntry()
{
  global $bibentry, $bibfile, $bibref, $pagename;
  if ($bibentry == false) PrintText($pagename, "%red%Invalid BibTex Entry: [" . $bibfile . ", " . $bibref . "]!");
  else
    {
      PrintText($pagename, $bibentry->getSolePageEntry());
    }
}

$BibEntries = array();

function GlobalRef($bib, $ref)
{
  return $bib . ":" . $ref;
}

function GetEntry($bib, $ref)
{
    global $BibEntries;
    $ref = trim($ref);
    $bib = trim($bib);
    $globalref = GlobalRef($bib, $ref);
    if ($BibEntries[$globalref] == false)
      ReadBibFile($bib);
    if ($BibEntries[$globalref] == false)
      return false; 
    return $BibEntries[$globalref];
}

function BibCite($bib, $ref)
{
  $entry = GetEntry($bib, $ref);
  if ($entry == false) return "%red%Invalid BibTex Entry!";
  return $entry->cite();
}

function CompleteBibEntry($bib, $ref)
{
  $entry = GetEntry($bib, $ref);
  if ($entry == false) return "%red%Invalid BibTex Entry!";
  return $entry->getCompleteEntry();
}

function BibSummary($bib, $ref)
{
  $entry = GetEntry($bib, $ref);
  if ($entry == false) return "%red%Invalid BibTex Entry!";
  return $entry->getSummary();
}

function ReadBibFile($bib)
{
  global $pagename;
  $file = fopen("uploads/" . FmtPageName('$Group', $pagename) . "/" . $bib, 'r');
  if ($file)
  {
    while(ReadBibEntry($bib, $file) != false);
    fclose($file);
  } 
}

function ReadBibEntry($fname, $file)
{
  global $BibEntries;

  // First reach the '@' entry
  do {
    $line = trim(fgets($file));
  } while ($line[0] != '@' && !feof($file));

  if (feof($file)) return false;

  $entrytype = strtoupper(trim(substr($line, 1, strpos($line, "{") - 1)));

  if ($entrytype == "COMMENT") return true;

  $entryname = trim(substr($line, strpos($line, "{") + 1, strrpos($line, ",") - strpos($line, "{") - 1));

  if ($entrytype == "ARTICLE") $entry = new Article($fname, $entryname);
  else if ($entrytype == "INPROCEEDINGS") $entry = new InProceedings($fname, $entryname);
  else if ($entrytype == "PHDTHESIS") $entry = new PhdThesis($fname, $entryname);
  else if ($entrytype == "MASTERSTHESIS") $entry = new MasterThesis($fname, $entryname);
  else if ($entrytype == "INCOLLECTION") $entry = new InCollection($fname, $entryname);
  else if ($entrytype == "BOOK") $entry = new Book($fname, $entryname);
  else if ($entrytype == "TECHREPORT") $entry = new TechReport($fname, $entryname);
  else $entry = new Misc($fname, $entryname);

  $line = trim(fgets($file));
  // Then read until we get to the closing bracket
  do {
    $value = "";
    if (feof($file)) return false;
    $key = strtoupper(trim(substr($line, 0, strpos($line, "=") - 1)));

    $val1 = trim(substr($line, strpos($line, "{") + 1));
    while ($val1[strlen($val1) - 1] != ',' || $val1[strlen($val1) - 2] != '}')
      {
	$value = $value . " " . $val1;
	$val1 = trim(fgets($file));
      }
    $value = $value . " " . $val1;
    $value = trim(substr($value, 0, strrpos($value, "}")));

    $entry->values[$key] = $value;
    $line = trim(fgets($file));
  } while ($line[0] != "}");

  $BibEntries[GlobalRef($fname, $entryname)] = $entry;
  return true;
}

?>
